# -*- encoding: utf-8 -*-
# Copyright (c) 2020 Stephen Bunn <stephen@bunn.io>
# ISC License <https://choosealicense.com/licenses/isc>

"""Predefined style (sty) chalk styles."""

from ..chalk import Chalk
from ..style import Style

bold = Chalk(style={Style.BOLD})
dim = Chalk(style={Style.DIM})
italic = Chalk(style={Style.ITALIC})
underline = Chalk(style={Style.UNDERLINE})
slow_blink = Chalk(style={Style.SLOW_BLINK})
rapid_blink = Chalk(style={Style.RAPID_BLINK})
reversed = Chalk(style={Style.REVERSED})
conceal = Chalk(style={Style.CONCEAL})
strikethrough = Chalk(style={Style.STRIKETHROUGH})
normal = Chalk(style={Style.NORMAL})



# -*- encoding: utf-8 -*-
# Copyright (c) 2020 Stephen Bunn <stephen@bunn.io>
# ISC License <https://choosealicense.com/licenses/isc>

"""Predefined background (bg) chalk colors."""

from ..chalk import Chalk
from ..color import Color

black = Chalk(background=Color.BLACK)
red = Chalk(background=Color.RED)
green = Chalk(background=Color.GREEN)
yellow = Chalk(background=Color.YELLOW)
blue = Chalk(background=Color.BLUE)
magenta = Chalk(background=Color.MAGENTA)
cyan = Chalk(background=Color.CYAN)
white = Chalk(background=Color.WHITE)

bright_black = Chalk(background=Color.BRIGHT_BLACK)
bright_red = Chalk(background=Color.BRIGHT_RED)
bright_green = Chalk(background=Color.BRIGHT_GREEN)
bright_yellow = Chalk(background=Color.BRIGHT_YELLOW)
bright_blue = Chalk(background=Color.BRIGHT_BLUE)
bright_magenta = Chalk(background=Color.MAGENTA)
bright_cyan = Chalk(background=Color.BRIGHT_CYAN)
bright_white = Chalk(background=Color.BRIGHT_WHITE)



# -*- encoding: utf-8 -*-
# Copyright (c) 2020 Stephen Bunn <stephen@bunn.io>
# ISC License <https://choosealicense.com/licenses/isc>

"""Contains shortcuts for quickly utilizing chalk.

Simply combine colors and styles using ``&`` or ``+`` to produce a style that can used
to format a string with ``|`` or ``+``.

Examples:
    >>> from chalky import bg, fg, sty
    >>> my_style = bg.red & fg.black & sty.bold
    >>> print(my_style | "I'm bold black on red text")
    I'm bold black on red text

Many chalk styles can be combined together:

Examples:
    >>> from chalky import sty
    >>> my_style = sty.bold & sty.underline
    >>> print(my_style | "I'm bold and underlined")
    I'm bold and underlined

The last applied foreground or background color will be used when applied to a string:

Examples:
    >>> from chalky import bg
    >>> my_style = bg.red & bg.blue  # BLUE will override RED when styling the string
    >>> print(my_style | "My background is BLUE")
    My background is BLUE
"""

from ..chalk import Chalk
from ..color import TrueColor
from . import bg, fg, sty


def rgb(red: int, green: int, blue: int, background: bool = False) -> Chalk:
    """Generate a new truecolor chalk from an RGB tuple.

    Args:
        red (int):
            The intensity of red (0-255).
        green (int):
            The intensity of green (0-255).
        blue (int):
            The intensity of blue (0-255).
        background (bool, optional):
            If ``True`` will generate the new chalk to be applied as a background color.
            Defaults to False.

    Returns:
        :class:`~.chalk.Chalk`:
            The new chalk instance.
    """

    color = TrueColor(red, green, blue)
    return Chalk(background=color) if background else Chalk(foreground=color)


def hex(hexcolor: str, background: bool = False) -> Chalk:
    """Generate a new truecolor chalk from a HEX color (``#ffffff``) string.

    Args:
        hexcolor (str):
            The hex color string.
        background (bool, optional):
            If ``True`` will generate the new chalk to be applied as background color.
            Defaults to False.

    Returns:
        :class:`~.chalk.Chalk`:
            The new chalk instance.
    """

    color = TrueColor.from_hex(hexcolor)
    return Chalk(background=color) if background else Chalk(foreground=color)


__all__ = [
    "fg",
    "bg",
    "sty",
    "rgb",
    "hex",
    "chain",
]



# -*- encoding: utf-8 -*-
# Copyright (c) 2020 Stephen Bunn <stephen@bunn.io>
# ISC License <https://choosealicense.com/licenses/isc>

"""Contains the actual implementation of interacting with a type of terminal buffer."""


import sys
from functools import lru_cache
from typing import Optional, TextIO

from .ansi import AnsiInterface
from .base import BaseInterface


@lru_cache()
def get_interface(io: Optional[TextIO] = None) -> BaseInterface:
    """Get the appropriate interface to interact with some terminal buffer.

    Examples:
        Get the default interface for interacting with the terminal buffer.
        This defaults to using :data:`sys.stdout`

        >>> from chalky.interface import get_interface
        >>> interface = get_interface()

        To get an interface using a different text io buffer, pass it in:

        >>> import sys
        >>> stderr_interface = get_interface(sys.stderr)

    Args:
        io (Optional[:class:`~typing.TextIO`], optional):
            The io to build an interface for.
            Defaults to :data:`sys.stdout`.

    Returns:
        :class:`~interface.base.BaseInterface`:
            The created interface for the given io buffer.
    """

    if not io:
        io = sys.stdout

    return AnsiInterface(io)


__all__ = ["get_interface"]



# -*- encoding: utf-8 -*-
# Copyright (c) 2020 Stephen Bunn <stephen@bunn.io>
# ISC License <https://choosealicense.com/licenses/isc>

"""Contains the available tools to define the colors that can be used for chalk."""

from __future__ import annotations

from dataclasses import dataclass
from enum import Enum
from typing import Tuple, Union

from .helpers import int_to_bytes


class Color(Enum):
    """Enum of the available colors that can be used to color some printable text.

    Attributes:
        BLACK
        RED
        GREEN
        YELLOW
        BLUE
        MAGENTA
        CYAN
        WHITE
        BRIGHT_BLACK:
            Otherwise known as gray.
        BRIGHT_RED
        BRIGHT_GREEN
        BRIGHT_YELLOW
        BRIGHT_BLUE
        BRIGHT_MAGENTA
        BRIGHT_CYAN
        BRIGHT_WHITE:
            Otherwise known as actual white.
    """

    BLACK = "black"
    RED = "red"
    GREEN = "green"
    YELLOW = "yellow"
    BLUE = "blue"
    MAGENTA = "magenta"
    CYAN = "cyan"
    WHITE = "white"

    BRIGHT_BLACK = "bright_black"
    BRIGHT_RED = "bright_red"
    BRIGHT_GREEN = "bright_green"
    BRIGHT_YELLOW = "bright_yellow"
    BRIGHT_BLUE = "bright_blue"
    BRIGHT_MAGENTA = "bright_magenta"
    BRIGHT_CYAN = "bright_cyan"
    BRIGHT_WHITE = "bright_white"


@dataclass
class TrueColor:
    """Describes a true color that can be displayed on compatible terminals.

    Parameters:
        red (int):
            The value of red (0-255).
        green (int):
            The value of green (0-255).
        blue (int):
            The value of blue (0-255).
    """

    red: int
    green: int
    blue: int

    def __hash__(self) -> int:
        """Generate a comparable hash for the current instance.

        Returns:
            int:
                The appropriate hash of the current instance.
        """

        return hash(f"{self.red}{self.green}{self.blue}")

    @classmethod
    def from_hex(cls, color: str) -> TrueColor:
        """Create an instance from a given hex color string.

        Args:
            color (str):
                The hex color string of the color to create.

        Raises:
            ValueError:
                If the given hex color string is not a length of 3 or 6

        Returns:
            TrueColor:
                The created instance.
        """

        color = color.lstrip("#")
        if len(color) not in (3, 6):
            raise ValueError(f"Hex color #{color!s} is not of length 3 or 6")

        if len(color) == 3:
            color = "".join([element * 2 for element in color])

        red, green, blue = [
            int(color[index : index + 2], 16) for index in range(0, len(color), 2)
        ]

        return cls(red=red, green=green, blue=blue)

    def to_bytes(self) -> Tuple[bytes, bytes, bytes]:
        """Convert the current color to a tuple of RGB bytes.

        Returns:
            Tuple[bytes, bytes, bytes]:
                The corresponding bytes of the current instance (red, green, blue).
        """

        return (
            int_to_bytes(self.red),
            int_to_bytes(self.green),
            int_to_bytes(self.blue),
        )


Color_T = Union[Color, TrueColor]



# -*- encoding: utf-8 -*-
# Copyright (c) 2020 Stephen Bunn <stephen@bunn.io>
# ISC License <https://choosealicense.com/licenses/isc>

"""Contains some miscellaneous helpers for the rest of the package."""

import os
import platform
import subprocess


def supports_posix() -> bool:
    """Check if the current machine supports basic posix.

    Returns:
        bool:
            True if the current machine is MacOSX or Linux.
    """

    return platform.system().lower() in (
        "darwin",
        "linux",
    )


def supports_truecolor() -> bool:
    """Attempt to check if the current terminal supports truecolor.

    Returns:
        bool:
            True if the current terminal supports truecolor, otherwise False.
    """

    colorterm = os.getenv("COLORTERM")
    if colorterm:
        return "truecolor" in colorterm.lower()

    term = os.getenv("TERM")
    if not term:
        return False

    term = term.lower()
    term_supported = any(
        value in term
        for value in (
            "256color",
            "24bit",
        )
    )
    if term_supported:
        return True

    if supports_posix():
        tput = subprocess.run(["tput", "colors"], capture_output=True)
        if tput and len(tput.stdout) > 0:
            return "256" in tput.stdout.decode("utf-8").split("\n")[0].lower()

    return False


def int_to_bytes(value: int) -> bytes:
    """Convert a given number to its representation in bytes.

    Args:
        value (int):
            The value to convert.

    Returns:
        bytes:
            The representation of the given number in bytes.
    """

    return bytes(str(value), "utf-8")



# -*- encoding: utf-8 -*-
# Copyright (c) 2020 Stephen Bunn <stephen@bunn.io>
# ISC License <https://choosealicense.com/licenses/isc>

"""Contains the available styles we can use for chalk."""

from enum import Enum


class Style(Enum):
    """Enum of the available styles that can be applied to some printable text.

    Attributes:
        RESET:
            Resets all styles and colors to the original terminal style.
        BOLD:
            Emphasizes the text by increasing the font weight.
        DIM:
            Dims the text color and sometimes decreases font weight.
        ITALIC:
            Italicize the text.
        UNDERLINE:
            Underlines the text (works in most modern terminals).
        SLOW_BLINK:
            Flash the text slowly (doesn't work in most modern terminals).
        RAPID_BLINK:
            Flash the text very quickly (doesn't work in most modern terminals).
        REVERSED:
            Reversed the current style of the terminal for the text.
        CONCEAL:
            Hide the text.
        STRIKETHROUGH:
            Draw a line through the text (works in most modern terminals).
        NORMAL:
            Normalizes the text for the current terminal.
    """

    RESET = "reset"
    BOLD = "bold"
    DIM = "dim"
    ITALIC = "italic"
    UNDERLINE = "underline"
    SLOW_BLINK = "slow_blink"
    RAPID_BLINK = "rapid_blink"
    REVERSED = "reversed"
    CONCEAL = "conceal"
    STRIKETHROUGH = "strikethrough"
    NORMAL = "normal"



# -*- encoding: utf-8 -*-
# Copyright (c) 2020 Stephen Bunn <stephen@bunn.io>
# ISC License <https://choosealicense.com/licenses/isc>

"""Simple ANSI terminal text coloring.

Compose multiple of the included chalk instances to produce a style that can be applied
directly to strings.
Compose multiple chalk instances together with ``&`` and apply it to a string
using ``|``:

.. code-block:: python
    :linenos:

    from chalky import sty, fg
    print(sty.bold & fg.green | "Hello, World!")
"""

from .chain import chain
from .chalk import Chalk
from .color import Color, TrueColor
from .constants import configure
from .shortcuts import bg, fg, hex, rgb, sty
from .style import Style

__all__ = [
    "Chalk",
    "Color",
    "TrueColor",
    "Style",
    "fg",
    "bg",
    "sty",
    "rgb",
    "hex",
    "chain",
    "configure",
]



# -*- encoding: utf-8 -*-
# Copyright (c) 2020 Stephen Bunn <stephen@bunn.io>
# ISC License <https://choosealicense.com/licenses/isc>

"""Contains the ANSI interface implementation."""

import time
from typing import Optional, Set

from ..color import Color, Color_T, TrueColor
from ..helpers import int_to_bytes
from ..style import Style
from .base import BaseInterface

ESC = b"\x1b"
CSI = ESC + b"["
OSC = ESC + b"]"
BEL = b"\a"

MODE_KEEP_HEAD = 0
MODE_KEEP_TAIL = 1
MODE_KEEP_NONE = 2

FOREGROUND_COLOR_MAP = {
    Color.BLACK: b"30",
    Color.RED: b"31",
    Color.GREEN: b"32",
    Color.YELLOW: b"33",
    Color.BLUE: b"34",
    Color.MAGENTA: b"35",
    Color.CYAN: b"36",
    Color.WHITE: b"37",
    Color.BRIGHT_BLACK: b"90",
    Color.BRIGHT_RED: b"91",
    Color.BRIGHT_GREEN: b"92",
    Color.BRIGHT_YELLOW: b"93",
    Color.BRIGHT_BLUE: b"94",
    Color.BRIGHT_MAGENTA: b"95",
    Color.BRIGHT_CYAN: b"96",
    Color.BRIGHT_WHITE: b"97",
}

BACKGROUND_COLOR_MAP = {
    Color.BLACK: b"40",
    Color.RED: b"41",
    Color.GREEN: b"42",
    Color.YELLOW: b"43",
    Color.BLUE: b"44",
    Color.MAGENTA: b"45",
    Color.CYAN: b"46",
    Color.WHITE: b"47",
    Color.BRIGHT_BLACK: b"100",
    Color.BRIGHT_RED: b"101",
    Color.BRIGHT_GREEN: b"102",
    Color.BRIGHT_YELLOW: b"103",
    Color.BRIGHT_BLUE: b"104",
    Color.BRIGHT_MAGENTA: b"105",
    Color.BRIGHT_CYAN: b"106",
    Color.BRIGHT_WHITE: b"107",
}

STYLE_MAP = {
    Style.RESET: b"0",
    Style.BOLD: b"1",
    Style.DIM: b"2",
    Style.ITALIC: b"3",
    Style.UNDERLINE: b"4",
    Style.SLOW_BLINK: b"5",
    Style.RAPID_BLINK: b"6",
    Style.REVERSED: b"7",
    Style.CONCEAL: b"8",
    Style.STRIKETHROUGH: b"9",
    Style.NORMAL: b"22",
}


def build_escape_sequence(code: bytes) -> bytes:
    """Build the escape sequence for some given sequence code.

    Args:
        code (bytes):
            The sequence code to build.

    Returns:
        bytes:
            The built escape sequence.
    """

    return CSI + code + b"m"


def build_style(style: Style) -> bytes:
    """Build the appropriate escape sequence for a given style.

    Args:
        style (:class:`~style.Style`):
            The style to build the escape sequence for.

    Returns:
        bytes:
            The proper escape sequence for styling text with the given style.
    """

    style_code = STYLE_MAP.get(style)
    if style_code is None:
        return b""

    return build_escape_sequence(style_code)


def build_truecolor(color: TrueColor, background: bool = False) -> bytes:
    """Build the appropriate escape sequence for a given truecolor.

    Args:
        color (:class:`~color.TrueColor`):
            The color to build the escape sequence for.
        background (bool, optional):
            True if the color should bve used as a background color.
            Defaults to False.

    Returns:
        bytes:
            The proper escape sequence for coloring text with the given truecolor.
    """

    red, green, blue = color.to_bytes()
    truecolor_code = (
        (b"48" if background else b"38") + b";2;" + red + b";" + green + b";" + blue
    )

    return build_escape_sequence(truecolor_code)


def build_color(color: Color_T, background: bool = False) -> bytes:
    """Build the appropriate escape sequence for a given color.

    Args:
        color (:attr:`~color.Color_T`):
            The color to build the escape sequence for.
        background (bool, optional):
            True if the color should be used as a background color.
            Defaults to False.

    Returns:
        bytes:
            The proper escape sequence for coloring text with the given color.
    """

    if isinstance(color, TrueColor):
        return build_truecolor(color, background=background)

    color_map = BACKGROUND_COLOR_MAP if background else FOREGROUND_COLOR_MAP
    color_code = color_map.get(color)
    if color_code is None:
        return b""

    return build_escape_sequence(color_code)


def get_clear_mode(keep_head: bool, keep_tail: bool) -> Optional[int]:
    """Get the appropriate clear mode for the ``keep_head`` and ``keep_tail`` params.

    Args:
        keep_head (bool):
            True if you would like to keep the head content behind the cursor.
        keep_tail (bool):
            True if you would like to keep the tail content after the cursor.

    Returns:
        Optional[int]:
            The appropriate clear mode if available.
            If ``None`` is provided, consider skipping clearing.
    """

    if keep_head and keep_tail:
        return None

    if keep_head:
        return MODE_KEEP_HEAD
    elif keep_tail:
        return MODE_KEEP_TAIL
    else:
        return MODE_KEEP_NONE


def build_clear_screen(mode: int) -> bytes:
    """Build the escape sequence to clear the entire screen.

    Various modes are available for clearing the current line in the terminal.
    These are more appropriately derived from :func:`~ansi.get_clear_mode`.

    * 0 - Clear from cursor to end of screen
    * 1 - Clear from cursor to beginning of screen
    * 2 - Clear entire screen

    Args:
        mode (int):
            The mode to use for clearing the entire screen.

    Returns:
        bytes:
            The proper escape sequence to clear the entire screen.
    """

    return CSI + int_to_bytes(mode) + b"J"


def build_clear_line(mode: int) -> bytes:
    """Build the escape sequence to clear the current line.

    Various modes are available for clearing the current line in the terminal.
    These are more appropriately derived from :func:`~ansi.get_clear_mode`.

    * 0 - Clear from cursor to end of line
    * 1 - Clear from cursor to beginning of line
    * 2 - Clear entire line

    Args:
        mode (int):
            The mode to use for clearing the current line.

    Returns:
        bytes:
            The proper escape sequence to clear the current line.
    """

    return CSI + int_to_bytes(mode) + b"K"


def build_reset() -> bytes:
    """Build the escape sequence to reset the current text style and color.

    Returns:
        bytes:
            The proper escape sequence to reset the terminal text style and color.
    """

    return build_escape_sequence(STYLE_MAP.get(Style.RESET, b"0"))


def build_position_cursor(x: int, y: int) -> bytes:
    """Build the appropriate escape sequence to control the terminal cursor position.

    Args:
        x (int):
            The new column position of the cursor (1-indexed).
        y (int):
            The new row position of the cursor (1-indexed).

    Raises:
        ValueError:
            When either the given row or column positions are less than or equal to 0.

    Returns:
        bytes:
            The appropriate escape sequence to move the terminal cursor.
    """

    if x <= 0 or y <= 0:
        raise ValueError(f"Cursor positions are 1-indexed, received x={x}, y={y}")

    return CSI + int_to_bytes(y) + b";" + int_to_bytes(x) + b"H"


def build_video(normal: bool) -> bytes:
    """Build the appropriate escape sequence to control the terminal video state.

    Args:
        normal (bool):
            True if the terminal video should be normal, False if it should be reversed.

    Returns:
        bytes:
            The proper escape sequence to control terminal video state.
    """

    return CSI + b"?5" + (b"l" if normal else b"h")


def build_cursor(show: bool) -> bytes:
    """Build the appropriate escape sequence to control the visibility of the cursor.

    Args:
        show (bool):
            True if the cursor should be visible, False if it should be hidden.

    Returns:
        bytes:
            The proper escape sequence to control cursor visibility.
    """

    return CSI + b"?25" + (b"h" if show else b"l")


def build_set_title(title: str) -> bytes:
    """Build the appropriate escape sequence for setting the terminal title.

    Args:
        title (str):
            The new title to set.

    Returns:
        bytes:
            The proper escape sequence to set the terminal title.
    """

    return OSC + b"2;" + title.encode("utf-8") + BEL


class AnsiInterface(BaseInterface):
    """The interface to control ANSI terminals."""

    def _write(self, content: bytes):
        """Write the given bytes and immediately flush them.

        Args:
            content (bytes):
                The bytes to write to the ANSI text io.
        """

        self.io.write(content.decode(self.io.encoding))
        self.io.flush()

    def clear_screen(
        self,
        reset_position: bool = True,
        keep_head: bool = False,
        keep_tail: bool = False,
    ):
        """Clear the current screen of the terminal.

        Args:
            reset_position (bool, optional):
                If True, will also reset the cursor position to the start.
                Defaults to True.
            keep_head (bool, optional):
                If True, will retain the text before the cursor position.
                Defaults to False.
            keep_tail (bool, optional):
                If True, will retain the text after the cursor position.
                Defaults to False.
        """

        mode = get_clear_mode(keep_head, keep_tail)
        if not mode:
            return

        clear_screen = build_clear_screen(mode)
        if reset_position:
            clear_screen += build_position_cursor(1, 1)

        self._write(clear_screen)

    def clear_line(
        self,
        keep_head: bool = False,
        keep_tail: bool = False,
    ):
        """Clear the current line of the terminal.

        Args:
            keep_head (bool, optional):
                If True, will retain the text before the cursor position.
                Defaults to False.
            keep_tail (bool, optional):
                If True, will retain the text after the cursor position.
                Defaults to False.
        """

        mode = get_clear_mode(keep_head, keep_tail)
        if not mode:
            return

        self._write(build_clear_line(mode))

    def set_title(self, title: str):
        """Set the current title of the terminal to the given value.

        Args:
            title (str): The new title of the terminal.
        """

        self._write(build_set_title(title))

    def reset(self):
        """Reset the current style of printed terminal text."""

        self._write(build_reset())

    def hide_cursor(self):
        """Hide the current terminal cursor."""

        self._write(build_cursor(False))

    def show_cursor(self):
        """Show the current terminal cursor."""

        self._write(build_cursor(True))

    def reverse_video(self):
        """Reverse the current terminal colors."""

        self._write(build_video(False))

    def normal_video(self):
        """Normalize the current terminal colors."""

        self._write(build_video(True))

    def flash(self, duration: float):
        """Flash the terminal by inverting the current terminal colors.

        .. important::
            This is a blocking call.
            Whatever duration you specify for the flash, the runtime will be blocked for
            that time.
            Unless, of course, you decide to handle making the command async yourself.

        Args:
            duration (float):
                The delay in seconds the flash should last.
        """

        self.reverse_video()
        time.sleep(duration)
        self.normal_video()

    def apply(
        self,
        value: str,
        style: Set[Style],
        background: Optional[Color_T],
        foreground: Optional[Color_T],
    ) -> str:
        """Apply some style and color to a string for ANSI terminals.

        Args:
            value (str):
                The string value to apply styles to.
            style (Set[:class:`~style.Style`]):
                The set of styles to apply to the given string.
            background (Optional[:class:`~color.Color_T`]):
                The background color to apply to the given string.
            foreground (Optional[:class:`~color.Color_T`]):
                The foreground color to apply to the given string.

        Returns:
            str: The styled string value.
        """

        escape_sequence = b"".join(map(build_style, style))
        if background:
            escape_sequence += build_color(background, background=True)
        if foreground:
            escape_sequence += build_color(foreground)

        return (
            escape_sequence.decode(self.io.encoding)
            + value
            + build_reset().decode(self.io.encoding)
        )



# -*- encoding: utf-8 -*-
# Copyright (c) 2020 Stephen Bunn <stephen@bunn.io>
# ISC License <https://choosealicense.com/licenses/isc>

"""Contains the base abstract interface to inherit from."""

import abc
from typing import Optional, Set, TextIO

from ..color import Color_T
from ..style import Style


class BaseInterface(abc.ABC):
    """The base abstract interface to inherit from."""

    def __init__(self, io: TextIO):
        """Initialize the interface with some text io buffer.

        Args:
            io (:class:`~typing.TextIO`):
                The text io buffer to write to.
        """

        self.io = io

    @abc.abstractmethod
    def clear_screen(
        self,
        reset_position: bool = True,
        keep_head: bool = False,
        keep_tail: bool = False,
    ):  # pragma: no cover
        """Clear the full screen of the terminal.

        Args:
            reset_position (bool, optional):
                If True, will set the cursor position to the starting point.
                Defaults to True.
            keep_head (bool, optional):
                If True, will retain the text before the cursor position.
                Defaults to False.
            keep_tail (bool, optional):
                If True, will reatin the text after the cursor position.
                Defaults to False.
        """

        ...

    @abc.abstractmethod
    def clear_line(
        self,
        keep_head: bool = False,
        keep_tail: bool = False,
    ):  # pragma: no cover
        """Clear the current line of the terminal.

        Args:
            keep_head (bool, optional):,
                If True, will retain the text before the cursor position.
                Defaults to False.
            keep_tail (bool, optional):
                If True, will retain the text after the cursor position.
                Defaults to False.
        """

        ...

    @abc.abstractmethod
    def set_title(self, value: str):  # pragma: no cover
        """Set the current title of the terminal.

        Args:
            value (str):
                The new title of the terminal.
        """

        ...

    @abc.abstractmethod
    def reset(self):  # pragma: no cover
        """Reset the current style of the terminal."""

        ...

    @abc.abstractmethod
    def hide_cursor(self):  # pragma: no cover
        """Hide the current terminal cursor."""

        ...

    @abc.abstractmethod
    def show_cursor(self):  # pragma: no cover
        """Show the current terminal cursor."""

        ...

    @abc.abstractmethod
    def flash(self, duration: float):  # pragma: no cover
        """Flash the terminal.

        Args:
            duration (float):
                The delay in seconds the flash should last.
        """

        ...

    @abc.abstractmethod
    def apply(
        self,
        value: str,
        style: Set[Style],
        background: Optional[Color_T],
        foreground: Optional[Color_T],
    ) -> str:  # pragma: no cover
        """Apply some styles and colors to a given string.

        Args:
            value (str):
                The string to apply styles and colors to.
            style (Set[:class:`~style.Style`]):
                The set of styles to apply to the string value.
            background (Optional[:class:`~color.Color_T`]):
                The background color to apply to the given string.
            foreground (Optional[:class:`~color.Color_T`]):
                The foreground color to apply to the given string.

        Returns:
            str:
                The newly styled string.
        """

        ...



# -*- encoding: utf-8 -*-
# Copyright (c) 2020 Stephen Bunn <stephen@bunn.io>
# ISC License <https://choosealicense.com/licenses/isc>

"""Contains the base chalk class that is used to group some style and colors."""

from __future__ import annotations

from dataclasses import dataclass, field
from typing import Any, Optional, Set, Union, overload

from .color import Color_T
from .constants import is_disabled
from .interface import get_interface
from .style import Style


@dataclass
class Chalk:
    """Describes the style and color to use for styling some printable text.

    Chalk can be composed using ``&`` and can be applied to strings with ``|``.
    You can create your own instance of chalk by setting your desired
    :class:`~style.Style` and :class:`~color.Color` when creating a new instance.

    Examples:
        Creating custom instances of chalk looks like the following:

        >>> my_chalk = Chalk(
        ...     style={Style.BOLD, Style.UNDERLINE},
        ...     foreground=Color.RED,
        ... )

        Composing two chalk instances can be done through either using ``&`` or ``+``:

        >>> bold_chalk = Chalk(style={Style.BOLD})
        >>> error_chalk = bold_chalk & Chalk(foreground=Color.RED)

        Using chalk instances to style strings can be done using either ``|`` or ``+``:

        >>> error_chalk | "Hello, World!"
        Hello, World!


    .. important::

        When composing two chalk instances together, the chalk *being* applied to the
        base chalk instance will override the foreground and background colors.
        This means that if you apply a new chalk with a different foreground color it
        will override the starting foreground color:

        >>> red = Chalk(foreground=Color.RED)
        >>> blue = Chalk(foreground=Color.BLUE)
        >>> assert blue == (red & blue)
    """

    style: Set[Style] = field(default_factory=set)
    foreground: Optional[Color_T] = field(default=None)
    background: Optional[Color_T] = field(default=None)

    def __and__(self, other: Chalk) -> Chalk:
        """Create a new chalk instance from the composition of two chalk.

        Args:
            other (:class:`~Chalk`):
                Another chalk to combine with the current chalk.

        Returns:
            :class:`~Chalk`:
                The newly created chalk instance.
        """

        return Chalk(
            style=self.style.union(other.style),
            foreground=other.foreground or self.foreground,
            background=other.background or self.background,
        )

    def __or__(self, value: Any) -> str:
        """Style some given string with the current chalk instance.

        .. tip::
            If a non-string value is provided, we will attempt to get the most
            appropriate string from the value by simply calling ``str(value)``.
            So if you are passing in an object, make sure to use an appropriate
            ``__str__`` or ``__repr__``.

        Args:
            value (~typing.Any):
                The value to apply the current chalk styles to.

        Returns:
            str:
                The newly styled string.
        """

        if is_disabled():
            return str(value)

        interface = get_interface()
        return interface.apply(
            value=str(value),
            style=self.style,
            background=self.background,
            foreground=self.foreground,
        )

    @overload
    def __add__(self, other: Chalk) -> Chalk:  # pragma: no cover
        """Handle applying chalk to other chalk instances.

        Args:
            other (:class:`~Chalk`):
                The chalk instance to apply to the current instance.

        Returns:
            :class:`~Chalk`:
                They newly created chalk instance.
        """

        ...

    @overload
    def __add__(self, other: str) -> str:  # pragma: no cover
        """Handle applying chalk to strings.

        Args:
            other (str):
                The string to style with the current chalk instance.

        Returns:
            str:
                They newly styled string.
        """

        ...

    def __add__(self, other: Union[Chalk, str]) -> Union[Chalk, str]:
        """Handle applying chalk instances to things.

        Args:
            other (Union[Chalk, str]):
                Either another chalk instance or a string.

        Returns:
            Union[Chalk, str]:
                The combined chalk instances or a styled string.
        """

        if isinstance(other, Chalk):
            return self & other

        return self | other

    def __call__(self, value: str) -> str:
        """Handle applying chalk instances to strings.

        Args:
            value (str):
                The string to apply the current styles to.

        Returns:
            str:
                The newly styled string.
        """

        return self | value

    @property
    def reverse(self) -> Chalk:
        """Color reverse of the current chalk instance.

        Returns:
            :class:`~Chalk`:
                The revsered chalk instance
        """

        return self & Chalk(style={Style.REVERSED})



# -*- encoding: utf-8 -*-
# Copyright (c) 2020 Stephen Bunn <stephen@bunn.io>
# ISC License <https://choosealicense.com/licenses/isc>

"""Contains package constants that modify the global functions of the package.

Utilize the :func:`~.constants.configure` method to quickly and easily disable all
future application of :class:`~.chalk.Chalk` to strings.

>>> from chalky import configure, fg
>>> configure(disable=True)
>>> print(fg.green | "I'm NOT green text")
"""


DISABLED = False


def is_disabled() -> bool:
    """Callable to evaluate the disabled conditional.

    Returns:
        bool:
            True if chalky is disabled, otherwise False.
    """

    return DISABLED


def configure(disable: bool = False):
    """Configure the global state of the chalky module.

    Args:
        disable (bool, optional):
            If True, will disable all future application of colors and styles.
            Defaults to False.
    """

    global DISABLED

    DISABLED = disable



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['chalky', 'chalky.interface', 'chalky.shortcuts']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'chalky',
    'version': '1.0.0',
    'description': 'Simple ANSI terminal text coloring',
    'long_description': '![Chalky](https://github.com/stephen-bunn/chalky/raw/master/docs/source/_static/assets/img/Chalky.png)\n\n[![Supported Versions](https://img.shields.io/pypi/pyversions/chalky.svg)](https://pypi.org/project/chalky/)\n[![Test Status](https://github.com/stephen-bunn/chalky/workflows/Test%20Package/badge.svg)](https://github.com/stephen-bunn/chalky)\n[![Documentation Status](https://readthedocs.org/projects/chalky/badge/?version=latest)](https://chalky.readthedocs.io/)\n[![Codecov](https://codecov.io/gh/stephen-bunn/chalky/branch/master/graph/badge.svg?token=G3KRpTeg5J)](https://codecov.io/gh/stephen-bunn/chalky)\n[![Code Style: Black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/ambv/black)\n\n> Simple ANSI terminal text coloring\n\nYet another terminal text coloring library…\n\nWhy? Because, I like certain things and I hate certain things about the currently\navailable solutions.\nThis here is my attempt to build an interface for simply applying ANSI escape sequences\nto strings that I enjoy and can update at my own free will.\nThat is it, there is nothing new or interesting that this packages adds.\nThanks 🎉\n\nFor more interesting details, please visit the\n[documentation](https://chalky.readthedocs.io/).\n\n## Style Composition\n\n```python\nfrom chalky import sty, fg\n\nmy_style = sty.bold & fg.red\nprint(my_style | "This is red on black")\nprint(my_style.reverse | "This is black on red")\n```\n\n![Basic Colors](https://github.com/stephen-bunn/chalky/raw/master/docs/source/_static/assets/img/basic.png)\n\n## Style Chaining\n\n```python\nfrom chalky import chain\n\nprint(chain.bold.green | "I\'m bold green text")\nprint(chain.white.bg.red.italic | "I\'m italic white text on a red background")\n```\n\n![Style Colors](https://github.com/stephen-bunn/chalky/raw/master/docs/source/_static/assets/img/chaining.png)\n\n## Truecolor\n\n```python\nfrom chalky import rgb, sty, hex\n\nprint(rgb(23, 255, 122) & sty.italic | "Truecolor as well")\nprint(sty.bold & hex("#ff02ff") | "More and more colors")\n```\n\n![True Colors](https://github.com/stephen-bunn/chalky/raw/master/docs/source/_static/assets/img/truecolor.png)\n\n## Disable Colors\n\n```python\nfrom chalky import configure, fg\n\nprint(fg.red | "I am red text")\nconfigure(disable=True)\nprint(fg.red | "I am NOT red text")\n```\n\n![Configure](https://github.com/stephen-bunn/chalky/raw/master/docs/source/_static/assets/img/configure.png)\n',
    'author': 'Stephen Bunn',
    'author_email': 'stephen@bunn.io',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/stephen-bunn/chalky',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



# -*- encoding: utf-8 -*-
# Copyright (c) 2020 Stephen Bunn <stephen@bunn.io>
# ISC License <https://choosealicense.com/licenses/isc>

"""Contains the chain class that can be used to quickly produce styles and colors."""

from __future__ import annotations

from dataclasses import dataclass, field
from typing import Union, overload

from .chalk import Chalk
from .color import Color, Color_T, TrueColor
from .style import Style


@dataclass
class Chain:
    """Quickly produce a chain of styles and colors that can be applied to a string.

    Parameters:
        chalk (Chalk):
            The container chalk instance that contains the current chain style.

    Examples:
        Chaining styles together should be fairly straightforward:

        >>> from chalk import chain
        >>> print(chain.bold.blue | "Bold blue text")
        >>> print(chain.black.bg.green.italic | "Italic black text on green background")

        Once a :class:`~.chain.Chain` instance is applied to a string, the styles are
        consumed and the chain instance is reset.
    """

    _chalk: Chalk = field(default_factory=Chalk)
    _background: bool = field(default=False)

    def __and__(self, other: Union[Chalk, Chain]) -> Chain:
        """Compose the chain with another chain or a chalk instance.

        Args:
            other (Union[Chalk, Chain]):
                Another :class:`~.shortcuts.chain.Chain` or :class:`~.chalk.Chalk`
                instance to compose with the current chain.

        Returns:
            :class:`~.shortcuts.chain.Chain`:
                The newly updated chain.
        """

        if isinstance(other, Chalk):
            self._chalk = self._chalk & other
            return self

        self._chalk = self._chalk & other._chalk
        return self

    @overload
    def __add__(
        self, other: Union[Chalk, Chain]
    ) -> Chain:  # noqa: D105 # pragma: no cover
        ...

    @overload
    def __add__(self, other: str) -> str:  # noqa: D105 # pragma: no cover
        ...

    def __add__(self, other: Union[Chalk, Chain, str]) -> Union[Chain, str]:
        """Handle applying chain instances to things.

        Args:
            other (Union[Chalk, Chain, str]):
                Either a chalk instance, a chain instance, or a string.

        Returns:
            Union[Chain, str]:
                The updated chain or the applied string.
        """

        if isinstance(other, str):
            return self | other

        return self & other

    def __or__(self, value: str) -> str:
        """Style some given string with the current chain.

        Args:
            value (str):
                THe string to apply the current chain to.

        Returns:
            str:
                The newly styled string.
        """

        applied = self._chalk | value
        self._reset()

        return applied

    def __call__(self, value: str) -> str:
        """Handle applying a chain to a strings.

        Args:
            value (str):
                The string to apply the current chain to.

        Returns:
            str:
                The newly styled string.
        """

        return self | value

    def _reset(self):
        self._chalk = Chalk()
        self._background = False

    def _handle_style(self, style: Style) -> Chain:
        self._chalk.style.add(style)

        return self

    def _handle_color(self, color: Color_T) -> Chain:
        if self._background:
            self._chalk.background = color
        else:
            self._chalk.foreground = color

        return self

    def rgb(self, red: int, green: int, blue: int) -> Chain:
        """Add a truecolor chalk from an RGB tuple.

        Args:
            red (int):
                The intensity of red (0-255).
            green (int):
                The intensity of green (0-255).
            blue (int):
                The intensity of blue (0-255).

        Returns:
            :class:`~.chain.Chain`:
                The newly updated chain.
        """

        return self._handle_color(TrueColor(red, green, blue))

    def hex(self, color: str) -> Chain:
        """Add a truecolor chalk from a hex string.

        Args:
            color (str):
                The hex color string (#ffffff)

        Returns:
            :class:`~.chain.Chain`:
                The newly updated chain.
        """

        return self._handle_color(TrueColor.from_hex(color))

    @property
    def chalk(self) -> Chalk:
        """Extract the currently built chalk instance.

        .. important::
            Consuming this property will reset the current chain's styles.
            We are assuming that if you need the :class:`~.chalk.Chalk`, you have
            finished constructing it through the chaining syntax.

        Returns:
            ~.chalk.Chalk:
                The current chalk instance from the chained styles and colors.
        """

        chalk = self._chalk
        self._reset()

        return chalk

    @property
    def bg(self) -> Chain:
        """Following colors will be applied as the background color."""

        self._background = True
        return self

    @property
    def fg(self) -> Chain:
        """Following colors will be applied as the foreground color."""

        self._background = False
        return self

    @property
    def bold(self) -> Chain:  # noqa: D102 # pragma: no cover
        return self._handle_style(Style.BOLD)

    @property
    def dim(self) -> Chain:  # noqa: D102 # pragma: no cover
        return self._handle_style(Style.DIM)

    @property
    def italic(self) -> Chain:  # noqa: D102 # pragma: no cover
        return self._handle_style(Style.ITALIC)

    @property
    def underline(self) -> Chain:  # noqa: D102 # pragma: no cover
        return self._handle_style(Style.UNDERLINE)

    @property
    def slow_blink(self) -> Chain:  # noqa: D102 # pragma: no cover
        return self._handle_style(Style.SLOW_BLINK)

    @property
    def rapid_blink(self) -> Chain:  # noqa: D102 # pragma: no cover
        return self._handle_style(Style.RAPID_BLINK)

    @property
    def reversed(self) -> Chain:  # noqa: D102 # pragma: no cover
        return self._handle_style(Style.REVERSED)

    @property
    def conceal(self) -> Chain:  # noqa: D102 # pragma: no cover
        return self._handle_style(Style.CONCEAL)

    @property
    def strikethrough(self) -> Chain:  # noqa: D102 # pragma: no cover
        return self._handle_style(Style.STRIKETHROUGH)

    @property
    def normal(self) -> Chain:  # noqa: D102 # pragma: no cover
        return self._handle_style(Style.NORMAL)

    @property
    def black(self) -> Chain:  # noqa: D102 # pragma: no cover
        return self._handle_color(Color.BLACK)

    @property
    def red(self) -> Chain:  # noqa: D102 # pragma: no cover
        return self._handle_color(Color.RED)

    @property
    def green(self) -> Chain:  # noqa: D102 # pragma: no cover
        return self._handle_color(Color.GREEN)

    @property
    def yellow(self) -> Chain:  # noqa: D102 # pragma: no cover
        return self._handle_color(Color.YELLOW)

    @property
    def blue(self) -> Chain:  # noqa: D102 # pragma: no cover
        return self._handle_color(Color.BLUE)

    @property
    def magenta(self) -> Chain:  # noqa: D102 # pragma: no cover
        return self._handle_color(Color.MAGENTA)

    @property
    def cyan(self) -> Chain:  # noqa: D102 # pragma: no cover
        return self._handle_color(Color.CYAN)

    @property
    def white(self) -> Chain:  # noqa: D102 # pragma: no cover
        return self._handle_color(Color.WHITE)

    @property
    def bright_black(self) -> Chain:  # noqa: D102 # pragma: no cover
        return self._handle_color(Color.BRIGHT_BLACK)

    @property
    def bright_red(self) -> Chain:  # noqa: D102 # pragma: no cover
        return self._handle_color(Color.BRIGHT_RED)

    @property
    def bright_green(self) -> Chain:  # noqa: D102 # pragma: no cover
        return self._handle_color(Color.BRIGHT_GREEN)

    @property
    def bright_yellow(self) -> Chain:  # noqa: D102 # pragma: no cover
        return self._handle_color(Color.BRIGHT_YELLOW)

    @property
    def bright_blue(self) -> Chain:  # noqa: D102 # pragma: no cover
        return self._handle_color(Color.BRIGHT_BLUE)

    @property
    def bright_magenta(self) -> Chain:  # noqa: D102 # pragma: no cover
        return self._handle_color(Color.BRIGHT_MAGENTA)

    @property
    def bright_cyan(self) -> Chain:  # noqa: D102 # pragma: no cover
        return self._handle_color(Color.BRIGHT_CYAN)

    @property
    def bright_white(self) -> Chain:  # noqa: D102 # pragma: no cover
        return self._handle_color(Color.BRIGHT_WHITE)


chain = Chain()
